Attribute VB_Name = "mGDI"
Public Enum COMPARE_METHODS
    eDIB = 0
    eDIBStrings = 1
    eTrueDIB = 2
    eTrueDIBStrings = 3
End Enum

Public Enum COMPARE_RESULTS
    ePicMatches
    eDiffType
    eDiffDimensions
    eDiffColorDepth
    eDiffBits
End Enum

Public Type BITMAPINFOHEADER '40 bytes
     biSize As Long
     biWidth As Long
     biHeight As Long
     biPlanes As Integer
     biBitCount As Integer
     biCompression As Long
     biSizeImage As Long
     biXPelsPerMeter As Long
     biYPelsPerMeter As Long
     biClrUsed As Long
     biClrImportant As Long
End Type

Public Type RGBQUAD
     rgbBlue As Byte
     rgbGreen As Byte
     rgbRed As Byte
     rgbReserved As Byte
End Type

Public Type BITMAPINFO
    bmiHeader As BITMAPINFOHEADER
    bmiColors As RGBQUAD
End Type

Public Type BITMAP
    bmType As Long
    bmWidth As Long
    bmHeight As Long
    bmWidthBytes As Long
    bmPlanes As Integer
    bmBitsPixel As Integer
    bmBits As Long
End Type

Private Type ICONINFO
   fIcon  As Long      'BOOL
   xHotspot As Long
   yHotspot As Long
   hbmMask As Long
   hbmColor As Long
End Type

Public Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (lpvDest As Any, lpvSource As Any, ByVal cbCopy As Long)
Public Declare Function GetDesktopWindow Lib "user32" () As Long
Public Declare Function CreateDCAsNull Lib "gdi32" Alias "CreateDCA" (ByVal lpDriverName As String, lpDeviceName As Any, lpOutput As Any, lpInitData As Any) As Long
Public Declare Function GetObjectAPI Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Public Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hDC As Long) As Long
Public Declare Function SelectObject Lib "gdi32" (ByVal hDC As Long, ByVal hObject As Long) As Long
Public Declare Function DeleteDC Lib "gdi32" (ByVal hDC As Long) As Long
Public Declare Function ReleaseDC Lib "user32" (ByVal hWnd As Long, ByVal hDC As Long) As Long

Public Declare Function GetDIBits Lib "gdi32" (ByVal aHDC As Long, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long, lpBits As Any, lpBI As Any, ByVal wUsage As Long) As Long
Public Declare Function GetTickCount Lib "kernel32" () As Long

Private Declare Function GetIconInfo Lib "user32" (ByVal hIcon As Long, piconinfo As ICONINFO) As Long

Public Const BI_RGB = 0&
Public Const DIB_RGB_COLORS = 0

Public Function CompareBitmaps(ByVal picFirst As StdPicture, ByVal picSecond As StdPicture, Optional ByVal method As COMPARE_METHODS = eTrueDIB) As COMPARE_RESULTS
   Dim bmp_1 As BITMAP, bmp_2 As BITMAP
   Dim abPicture_1() As Byte, abPicture_2() As Byte
   Dim hBitmap_1 As Long, hBitmap_2 As Long
   Dim picInfo As ICONINFO
   Dim bResult As Boolean

   If picFirst.Type = vbPicTypeIcon Then
      Call GetIconInfo(picFirst.Handle, picInfo)
      hBitmap_1 = picInfo.hbmColor
   Else
      hBitmap_1 = picFirst.Handle
   End If
   If picSecond.Type = vbPicTypeIcon Then
      Call GetIconInfo(picSecond.Handle, picInfo)
      hBitmap_2 = picInfo.hbmColor
   Else
      hBitmap_2 = picSecond.Handle
   End If
   
   Call GetObjectAPI(hBitmap_1, Len(bmp_1), bmp_1)
   Call GetObjectAPI(hBitmap_2, Len(bmp_2), bmp_2)
   If (bmp_1.bmWidth <> bmp_2.bmWidth) Or (bmp_1.bmHeight <> bmp_2.bmHeight) Then
      CompareBitmaps = eDiffDimensions: Exit Function
   End If
   If bmp_1.bmType <> bmp_2.bmType Then
      CompareBitmaps = eDiffType: Exit Function
   End If
   If (bmp_1.bmBitsPixel <> bmp_2.bmBitsPixel) Or (bmp_1.bmPlanes <> bmp_2.bmPlanes) Then
      CompareBitmaps = eDiffColorDepth: Exit Function
   End If
   Select Case method
          Case eDIB
               Call DIBFromPicture(hBitmap_1, abPicture_1)
               Call DIBFromPicture(hBitmap_2, abPicture_2)
               bResult = CompareBytes(abPicture_1, abPicture_2)
          Case eDIBStrings
               bResult = (DIBStringFromPicture(hBitmap_1) = DIBStringFromPicture(hBitmap_2))
          Case eTrueDIB
               Call TrueDIBFromPicture(hBitmap_1, abPicture_1)
               Call TrueDIBFromPicture(hBitmap_2, abPicture_2)
               bResult = CompareBytes(abPicture_1, abPicture_2)
          Case eTrueDIBStrings
               bResult = (TrueDIBStringFromPicture(hBitmap_1) = TrueDIBStringFromPicture(hBitmap_2))
          Case Else
               MsgBox "Unknown method!" & method, vbCritical, "Compare error"
   End Select
   If bResult Then
      CompareBitmaps = ePicMatches
   Else
      CompareBitmaps = eDiffBits
   End If
End Function

Private Sub DIBFromPicture(ByVal hBitmap As Long, abPicture() As Byte)
   Dim hdcScrn As Long, hwndScrn As Long
   Dim TempDC As Long, hOldBmp As Long
   Dim BuffSize As Long
   Dim bmp As BITMAP
   Dim bih As BITMAPINFOHEADER
   Dim biArray() As Byte
   Dim sPicture As String
   
   hwndScrn = GetDesktopWindow
   hdcScrn = CreateDCAsNull("DISPLAY", ByVal 0&, ByVal 0&, ByVal 0&)
   TempDC = CreateCompatibleDC(hdcScrn)
   hOldBmp = SelectObject(TempDC, hBitmap)
   bih.biSize = Len(bih)
   Call GetDIBits(TempDC, hBitmap, 0, 0, ByVal 0, bih, DIB_RGB_COLORS)
   bih.biCompression = BI_RGB
   BuffSize = bih.biWidth
   Select Case bih.biBitCount
       Case 1
            BuffSize = Int((BuffSize + 7) / 8)
            ReDim biArray(Len(bih) + 4 * 2 - 1)
       Case 4
            BuffSize = Int((BuffSize + 1) / 2)
            ReDim biArray(Len(bih) + 4 * 16 - 1)
       Case 8
            BuffSize = BuffSize
            ReDim biArray(Len(bih) + 4 * 256 - 1)
       Case 16
            BuffSize = BuffSize * 2
            ReDim biArray(Len(bih) + 4 - 1)
       Case 24
            BuffSize = BuffSize * 3
            ReDim biArray(Len(bih) + 4 - 1)
       Case 32
            BuffSize = BuffSize * 3
            ReDim biArray(Len(bih) + 4 * 3 - 1)
   End Select
   BuffSize = (Int((BuffSize + 3) / 4)) * 4
   BuffSize = BuffSize * bih.biHeight
   ReDim abPicture(BuffSize - 1)
   CopyMemory biArray(0), bih, Len(bih)
   Call GetDIBits(TempDC, hBitmap, 0, bih.biHeight, abPicture(0), biArray(0), DIB_RGB_COLORS)
   SelectObject TempDC, OldBitmap
   DeleteDC TempDC
   ReleaseDC hwndScrn, hdcScrn
End Sub

Private Sub TrueDIBFromPicture(ByVal hBitmap As Long, abPicture() As Byte)
   Dim hdcScrn As Long, hwndScrn As Long
   Dim TempDC As Long, hOldBmp As Long, imgSize As Long
   Dim bmp As BITMAP
   Dim BI As BITMAPINFO
   Call GetObjectAPI(hBitmap, Len(bmp), bmp)
   With BI.bmiHeader
       .biSize = Len(BI.bmiHeader)
       .biWidth = bmp.bmWidth
       .biHeight = bmp.bmHeight
       .biPlanes = 1
       .biBitCount = 24
       .biCompression = BI_RGB
       imgSize = BytesPerScanLine(.biWidth) * .biHeight
       .biSizeImage = imgSize
       ReDim abPicture(imgSize - 1)
   End With
   hwndScrn = GetDesktopWindow
   hdcScrn = CreateDCAsNull("DISPLAY", ByVal 0&, ByVal 0&, ByVal 0&)
   TempDC = CreateCompatibleDC(hdcScrn)
   hOldBmp = SelectObject(TempDC, hBitmap)
   Call GetDIBits(TempDC, hBitmap, 0, BI.bmiHeader.biHeight, abPicture(0), BI, DIB_RGB_COLORS)
   SelectObject TempDC, OldBitmap
   DeleteDC TempDC
   ReleaseDC hwndScrn, hdcScrn
End Sub

Private Function DIBStringFromPicture(ByVal hBitmap As Long) As String
   Dim hdcScrn As Long, hwndScrn As Long
   Dim TempDC As Long, hOldBmp As Long
   Dim BuffSize As Long
   Dim bmp As BITMAP
   Dim bih As BITMAPINFOHEADER
   Dim biArray() As Byte
   Dim sPicture As String
   
   hwndScrn = GetDesktopWindow
   hdcScrn = CreateDCAsNull("DISPLAY", ByVal 0&, ByVal 0&, ByVal 0&)
   TempDC = CreateCompatibleDC(hdcScrn)
   hOldBmp = SelectObject(TempDC, hBitmap)
   bih.biSize = Len(bih)
   Call GetDIBits(TempDC, hBitmap, 0, 0, ByVal 0, bih, DIB_RGB_COLORS)
   bih.biCompression = BI_RGB
   BuffSize = bih.biWidth
   Select Case bih.biBitCount
       Case 1
            BuffSize = Int((BuffSize + 7) / 8)
            ReDim biArray(Len(bih) + 4 * 2 - 1)
       Case 4
            BuffSize = Int((BuffSize + 1) / 2)
            ReDim biArray(Len(bih) + 4 * 16 - 1)
       Case 8
            BuffSize = BuffSize
            ReDim biArray(Len(bih) + 4 * 256 - 1)
       Case 16
            BuffSize = BuffSize * 2
            ReDim biArray(Len(bih) + 4 - 1)
       Case 24
            BuffSize = BuffSize * 3
            ReDim biArray(Len(bih) + 4 - 1)
       Case 32
            BuffSize = BuffSize * 3
            ReDim biArray(Len(bih) + 4 * 3 - 1)
   End Select
   BuffSize = (Int((BuffSize + 3) / 4)) * 4
   BuffSize = BuffSize * bih.biHeight
   sPicture = String(BuffSize, 0)
   CopyMemory biArray(0), bih, Len(bih)
   Call GetDIBits(TempDC, hBitmap, 0, bih.biHeight, ByVal sPicture, biArray(0), DIB_RGB_COLORS)
   DIBStringFromPicture = sPicture
   SelectObject TempDC, OldBitmap
   DeleteDC TempDC
   ReleaseDC hwndScrn, hdcScrn
End Function

Private Function TrueDIBStringFromPicture(ByVal hBitmap As Long) As String
   Dim hdcScrn As Long, hwndScrn As Long
   Dim TempDC As Long, hOldBmp As Long, imgSize As Long
   Dim bmp As BITMAP
   Dim BI As BITMAPINFO
   Dim sPicture As String
   
   Call GetObjectAPI(hBitmap, Len(bmp), bmp)
   With BI.bmiHeader
       .biSize = Len(BI.bmiHeader)
       .biWidth = bmp.bmWidth
       .biHeight = bmp.bmHeight
       .biPlanes = 1
       .biBitCount = 24
       .biCompression = BI_RGB
       imgSize = BytesPerScanLine(.biWidth) * .biHeight
       .biSizeImage = imgSize
       sPicture = String(imgSize, 0)
   End With
   hwndScrn = GetDesktopWindow
   hdcScrn = CreateDCAsNull("DISPLAY", ByVal 0&, ByVal 0&, ByVal 0&)
   TempDC = CreateCompatibleDC(hdcScrn)
   hOldBmp = SelectObject(TempDC, hBitmap)
   Call GetDIBits(TempDC, hBitmap, 0, BI.bmiHeader.biHeight, ByVal sPicture, BI, DIB_RGB_COLORS)
   TrueDIBStringFromPicture = sPicture
   SelectObject TempDC, OldBitmap
   DeleteDC TempDC
   ReleaseDC hwndScrn, hdcScrn
End Function

Private Function BytesPerScanLine(ByVal lWidth As Long) As Long
    BytesPerScanLine = (lWidth * 3 + 3) And &HFFFFFFFC
End Function

Private Function CompareBytes(abPic_1() As Byte, abPic_2() As Byte) As Boolean
    Dim xMax As Long, yMax As Long
    Dim x As Long, y As Long
    xMax = UBound(abPic_1, 1)
    For x = 0 To xMax
        If abPic_1(x) <> abPic_2(x) Then
           Exit Function
        End If
    Next x
    CompareBytes = True
End Function
